@@ -11,29 +11,24 @@ extern void sepol_debug(int on);
 #define SEPOL_MSG_WARN 2
 #define SEPOL_MSG_INFO 3
 
-extern int sepol_msg_get_level(
-	sepol_handle_t* handle);
+extern int sepol_msg_get_level(sepol_handle_t * handle);
 
-extern const char* sepol_msg_get_channel(
-	sepol_handle_t* handle);
+extern const char *sepol_msg_get_channel(sepol_handle_t * handle);
 
-extern const char* sepol_msg_get_fname(
-	sepol_handle_t* handle);
+extern const char *sepol_msg_get_fname(sepol_handle_t * handle);
 
 /* Set the messaging callback. 
  * By the default, the callback will print
  * the message on standard output, in a 
  * particular format. Passing NULL here
  * indicates that messaging should be suppressed */
-extern void sepol_msg_set_callback(
-	sepol_handle_t* handle,
+extern void sepol_msg_set_callback(sepol_handle_t * handle,
 #ifdef __GNUC__
-	__attribute__ ((format (printf, 3, 4)))
+				   __attribute__ ((format(printf, 3, 4)))
+#endif
+				   void (*msg_callback) (void *varg,
+							 sepol_handle_t *
+							 handle,
+							 const char *fmt, ...),
+				   void *msg_callback_arg);
 #endif
-	void (*msg_callback) (
-		void* varg,
-		sepol_handle_t* handle,
-		const char* fmt,
-		...),
-	void* msg_callback_arg);
-#endif 
